@@ -25,7 +25,7 @@
 #include <libanjuta/interfaces/ianjuta-document-manager.h>
 #include <libanjuta/interfaces/ianjuta-message-manager.h>
 #include <libanjuta/interfaces/ianjuta-file-manager.h>
-#include <libanjuta/plugins.h>
+#include <libanjuta/interfaces/ianjuta-preferences.h>
 #include <libanjuta/anjuta-debug.h>
 
 #include "plugin.h"
@@ -38,7 +38,7 @@ static gpointer parent_class;
 
 static void refresh (GtkAction *action, FileManagerPlugin *plugin)
 {
-	fv_refresh (plugin);
+	fv_refresh (plugin, TRUE);
 }
 
 static GtkActionEntry popup_actions[] = 
@@ -63,18 +63,18 @@ set_default_root_directory (FileManagerPlugin *fv)
 	{
 		fv_set_root (fv, "/");
 	}
-	fv_refresh (fv);
+	fv_refresh (fv, FALSE);
 }
 
 static void
 on_gconf_notify_prefs (GConfClient *gclient, guint cnxn_id,
 					   GConfEntry *entry, gpointer user_data)
 {
-	FileManagerPlugin *fv = (FileManagerPlugin*)user_data;
+	FileManagerPlugin *fv = ANJUTA_PLUGIN_FILE_MANAGER (user_data);
 	if (fv->project_is_loaded == FALSE)
 		set_default_root_directory (fv);
 	else
-		fv_refresh (fv);
+		fv_refresh (fv, TRUE);
 }
 
 #define REGISTER_NOTIFY(key, func) \
@@ -88,6 +88,7 @@ prefs_init (FileManagerPlugin *fv)
 	guint notify_id;
 	REGISTER_NOTIFY ("root.dir", on_gconf_notify_prefs);
 	REGISTER_NOTIFY ("filter.dir.ignore.hidden", on_gconf_notify_prefs);
+	REGISTER_NOTIFY ("filter.dir.ignore.pattern", on_gconf_notify_prefs);
 	REGISTER_NOTIFY ("filter.dir.match", on_gconf_notify_prefs);
 	REGISTER_NOTIFY ("filter.dir.unmatch", on_gconf_notify_prefs);
 	REGISTER_NOTIFY ("filter.file.ignore.hidden", on_gconf_notify_prefs);
@@ -117,7 +118,7 @@ project_root_added (AnjutaPlugin *plugin, const gchar *name,
 	FileManagerPlugin *fm_plugin;
 	const gchar *root_uri;
 
-	fm_plugin = (FileManagerPlugin *)plugin;
+	fm_plugin = ANJUTA_PLUGIN_FILE_MANAGER (plugin);
 	root_uri = g_value_get_string (value);
 	if (root_uri)
 	{
@@ -125,8 +126,8 @@ project_root_added (AnjutaPlugin *plugin, const gchar *name,
 		if (root_dir)
 		{
 			fv_set_root (fm_plugin, root_dir);
-			fv_refresh (fm_plugin);
-			((FileManagerPlugin *)plugin)->project_is_loaded = TRUE;
+			fv_refresh (fm_plugin, FALSE);
+			ANJUTA_PLUGIN_FILE_MANAGER (plugin)->project_is_loaded = TRUE;
 		}
 		else
 			set_default_root_directory (fm_plugin);
@@ -140,31 +141,34 @@ static void
 project_root_removed (AnjutaPlugin *plugin, const gchar *name,
 					  gpointer user_data)
 {
-	set_default_root_directory ((FileManagerPlugin *)plugin);
-	((FileManagerPlugin *)plugin)->project_is_loaded = FALSE;
+	set_default_root_directory (ANJUTA_PLUGIN_FILE_MANAGER (plugin));
+	ANJUTA_PLUGIN_FILE_MANAGER (plugin)->project_is_loaded = FALSE;
 }
 
 static gboolean
 activate_plugin (AnjutaPlugin *plugin)
 {
-	GladeXML *gxml;
 	FileManagerPlugin *fm_plugin;
 	gboolean initialized = FALSE;
 	
 	DEBUG_PRINT ("FileManagerPlugin: Activating File Manager plugin ...");
 	
-	fm_plugin = (FileManagerPlugin*) plugin;
+	fm_plugin = ANJUTA_PLUGIN_FILE_MANAGER (plugin);
 	fm_plugin->ui = anjuta_shell_get_ui (plugin->shell, NULL);
 	fm_plugin->prefs = anjuta_shell_get_preferences (plugin->shell, NULL);
 	fv_init (fm_plugin);
 	
+	fm_plugin->idle_id = 0;
+	fm_plugin->nodes_to_expand = NULL;
+	
 	/* Add action group */
 	fm_plugin->action_group = 
 		anjuta_ui_add_action_group_entries (fm_plugin->ui,
 											"ActionGroupFileManager",
 											_("File manager popup actions"),
 											popup_actions, 1,
-											GETTEXT_PACKAGE, plugin);
+											GETTEXT_PACKAGE, FALSE,
+											plugin);
 	/* Add UI */
 	fm_plugin->merge_id = 
 		anjuta_ui_merge (fm_plugin->ui, UI_FILE);
@@ -173,17 +177,6 @@ activate_plugin (AnjutaPlugin *plugin)
 	anjuta_shell_add_widget (plugin->shell, fm_plugin->scrolledwindow,
 							 "AnjutaFileManager", _("Files"), GTK_STOCK_OPEN,
 							 ANJUTA_SHELL_PLACEMENT_LEFT, NULL);
-	if (!initialized)
-	{
-		/* Add preferences */
-		gxml = glade_xml_new (PREFS_GLADE, "dialog.file.filter", NULL);
-		
-		anjuta_preferences_add_page (fm_plugin->prefs,
-									gxml, "File Manager", ICON_FILE);
-		g_object_unref (G_OBJECT (gxml));
-	}
-	prefs_init (fm_plugin);
-	on_gconf_notify_prefs (NULL, 0, NULL, fm_plugin);
 	
 	/* set up project directory watch */
 	fm_plugin->root_watch_id = anjuta_plugin_add_watch (plugin,
@@ -200,15 +193,11 @@ deactivate_plugin (AnjutaPlugin *plugin)
 	GtkWidget *widget_to_remove;
 	FileManagerPlugin *fm_plugin;
 	
-	fm_plugin = (FileManagerPlugin*) plugin;
-	prefs_finalize (fm_plugin);
+	fm_plugin = ANJUTA_PLUGIN_FILE_MANAGER (plugin);
 	
 	/* Remove watches */
 	anjuta_plugin_remove_watch (plugin, fm_plugin->root_watch_id, FALSE);
 	
-	/* Remove preferences */
-	/* FIXME: */
-	
 	/* Finalize it first so that we release our refs */
 	widget_to_remove = fm_plugin->scrolledwindow;
 	fv_finalize(fm_plugin);
@@ -236,7 +225,7 @@ dispose (GObject *obj)
 static void
 file_manager_plugin_instance_init (GObject *obj)
 {
-	FileManagerPlugin *plugin = (FileManagerPlugin*) obj;
+	FileManagerPlugin *plugin = ANJUTA_PLUGIN_FILE_MANAGER (obj);
 	plugin->tree = NULL;
 	plugin->scrolledwindow = NULL;
 	plugin->top_dir = NULL;
@@ -261,7 +250,7 @@ static void
 ifile_manager_set_root (IAnjutaFileManager *file_manager,
 						const gchar *root, GError **err)
 {
-	fv_set_root ((FileManagerPlugin*) file_manager, root);
+	fv_set_root (ANJUTA_PLUGIN_FILE_MANAGER (file_manager), root);
 }
 
 static void
@@ -273,7 +262,7 @@ ifile_manager_set_selected (IAnjutaFileManager *file_manager,
 static gchar*
 ifile_manager_get_selected (IAnjutaFileManager *file_manager, GError **err)
 {
-	return fv_get_selected_file_path((FileManagerPlugin*) file_manager);
+	return fv_get_selected_file_path(ANJUTA_PLUGIN_FILE_MANAGER (file_manager));
 }
 
 static void
@@ -284,8 +273,38 @@ ifile_manager_iface_init(IAnjutaFileManagerIface *iface)
 	iface->set_selected = ifile_manager_set_selected;
 }
 
+static void
+ipreferences_merge(IAnjutaPreferences* ipref, AnjutaPreferences* prefs, GError** e)
+{
+	FileManagerPlugin* fm_plugin = ANJUTA_PLUGIN_FILE_MANAGER (ipref);
+	/* Add preferences */
+	GladeXML *gxml = glade_xml_new (PREFS_GLADE, "dialog.file.filter", NULL);
+		
+	anjuta_preferences_add_page (prefs,
+									gxml, "File Manager", _("File Manager"), ICON_FILE);
+	g_object_unref (G_OBJECT (gxml));
+	prefs_init (fm_plugin);
+	on_gconf_notify_prefs (NULL, 0, NULL, fm_plugin);
+}
+
+static void
+ipreferences_unmerge(IAnjutaPreferences* ipref, AnjutaPreferences* prefs, GError** e)
+{
+	FileManagerPlugin* fm_plugin = ANJUTA_PLUGIN_FILE_MANAGER (ipref);
+	prefs_finalize (fm_plugin);
+	anjuta_preferences_dialog_remove_page(ANJUTA_PREFERENCES_DIALOG(prefs), _("File Manager"));
+}
+
+static void
+ipreferences_iface_init(IAnjutaPreferencesIface* iface)
+{
+	iface->merge = ipreferences_merge;
+	iface->unmerge = ipreferences_unmerge;	
+}
+
 ANJUTA_PLUGIN_BEGIN (FileManagerPlugin, file_manager_plugin);
 ANJUTA_PLUGIN_ADD_INTERFACE (ifile_manager, IANJUTA_TYPE_FILE_MANAGER);
+ANJUTA_PLUGIN_ADD_INTERFACE (ipreferences, IANJUTA_TYPE_PREFERENCES);
 ANJUTA_PLUGIN_END;
 
 ANJUTA_SIMPLE_PLUGIN (FileManagerPlugin, file_manager_plugin);
