@@ -1,3 +1,4 @@
+/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
 /****************************************************************************
  *            sourceview.c
  *
@@ -26,14 +27,16 @@
 #include <libanjuta/anjuta-preferences.h>
 #include <libanjuta/interfaces/ianjuta-file.h>
 #include <libanjuta/interfaces/ianjuta-file-savable.h>
-#include <libanjuta/interfaces/ianjuta-editor.h>
 #include <libanjuta/interfaces/ianjuta-markable.h>
 #include <libanjuta/interfaces/ianjuta-indicable.h>
+#include <libanjuta/interfaces/ianjuta-bookmark.h>
+#include <libanjuta/interfaces/ianjuta-print.h>
+#include <libanjuta/interfaces/ianjuta-language-support.h>
+#include <libanjuta/interfaces/ianjuta-editor.h>
 #include <libanjuta/interfaces/ianjuta-editor-selection.h>
 #include <libanjuta/interfaces/ianjuta-editor-assist.h>
 #include <libanjuta/interfaces/ianjuta-editor-convert.h>
-#include <libanjuta/interfaces/ianjuta-bookmark.h>
-#include <libanjuta/interfaces/ianjuta-print.h>
+#include <libanjuta/interfaces/ianjuta-editor-language.h>
 
 #include <libgnomevfs/gnome-vfs-init.h>
 #include <libgnomevfs/gnome-vfs-mime-utils.h>
@@ -59,6 +62,7 @@
 #include "sourceview-scope.h"
 #include "sourceview-args.h"
 #include "sourceview-print.h"
+#include "sourceview-cell.h"
 #include "plugin.h"
 
 #define FORWARD 	0
@@ -71,15 +75,17 @@ static void sourceview_instance_init(Sourceview *sv);
 static void sourceview_finalize(GObject *object);
 static GObjectClass *parent_class = NULL;
 
-static void sourceview_add_monitor(Sourceview* sv);
+static gboolean sourceview_add_monitor(Sourceview* sv);
 
 /* Callbacks */
 
-static void on_document_char_added(AnjutaDocument* buffer, gchar character, Sourceview* sv)
+/* Called when a character is added */
+static void on_document_char_added(AnjutaView* view, gint pos,
+								   gchar ch,
+								   Sourceview* sv)
 {
-	/* FIXME: Pass correct character */
-	g_signal_emit_by_name(G_OBJECT(sv), "char_added",
-		ianjuta_editor_get_position(IANJUTA_EDITOR(sv), NULL), character);
+	DEBUG_PRINT("char-added: %c", ch);
+	g_signal_emit_by_name(G_OBJECT(sv), "char_added", pos, ch);
 }
 
 /* Called whenever the document is changed */
@@ -124,45 +130,22 @@ sourceview_remove_monitor(Sourceview* sv)
 	}
 }
 
-/* VFS-Monitor Callback */
-static void
-on_sourceview_uri_changed (GnomeVFSMonitorHandle *handle,
-							const gchar *monitor_uri,
-							const gchar *info_uri,
-							GnomeVFSMonitorEventType event_type,
-							gpointer user_data)
+static gboolean
+on_sourceview_uri_changed_prompt (Sourceview* sv)
 {
 	GtkWidget *dlg;
+	GtkWidget *parent;
 	gchar *buff;
-	Sourceview *sv = ANJUTA_SOURCEVIEW (user_data);
-	
-	DEBUG_PRINT ("File changed!!!");
-	
-	if (!(event_type == GNOME_VFS_MONITOR_EVENT_CHANGED ||
-		  event_type == GNOME_VFS_MONITOR_EVENT_CREATED))
-		return;
-	
-	sourceview_remove_monitor(sv);
-	
-	if (!anjuta_util_diff(anjuta_document_get_uri(sv->priv->document), 
-						  ianjuta_editor_get_text(IANJUTA_EDITOR(sv),
-						  0, ianjuta_editor_get_length(IANJUTA_EDITOR(sv), NULL),
-							NULL)))
-	{
-		sourceview_add_monitor(sv);
-		return;
-	}
-	
-	if (strcmp (monitor_uri, info_uri) != 0)
-		return;
 	
 	buff =
 		g_strdup_printf (_
 						 ("The file '%s' on the disk is more recent than\n"
 						  "the current buffer.\nDo you want to reload it?"),
-						 g_basename(anjuta_document_get_uri(sv->priv->document)));
+						 ianjuta_editor_get_filename(IANJUTA_EDITOR(sv), NULL));
 	
-	dlg = gtk_message_dialog_new (NULL,
+	parent = gtk_widget_get_toplevel (GTK_WIDGET (sv));
+	
+	dlg = gtk_message_dialog_new (GTK_WINDOW (parent),
 								  GTK_DIALOG_DESTROY_WITH_PARENT,
 								  GTK_MESSAGE_WARNING,
 								  GTK_BUTTONS_NONE, buff);
@@ -175,33 +158,66 @@ on_sourceview_uri_changed (GnomeVFSMonitorHandle *handle,
 								   GTK_RESPONSE_YES);
 	g_free (buff);
 	
-	g_signal_connect (dlg, "response",
+	gtk_window_set_transient_for (GTK_WINDOW (dlg),
+								  GTK_WINDOW (parent));
+	
+	g_signal_connect (G_OBJECT(dlg), "response",
 					  G_CALLBACK (on_reload_dialog_response),
 					  sv);
-	g_signal_connect_swapped (dlg, "delete-event",
+	gtk_widget_show (dlg);
+
+	g_signal_connect_swapped (G_OBJECT(dlg), "delete-event",
 					  G_CALLBACK (gtk_widget_destroy),
 					  dlg);
-	gtk_widget_show (dlg);
+	
+	return FALSE;
 }
 
-static void 
+static void
+on_sourceview_uri_changed (GnomeVFSMonitorHandle *handle,
+							const gchar *monitor_uri,
+							const gchar *info_uri,
+							GnomeVFSMonitorEventType event_type,
+							gpointer user_data)
+{
+	Sourceview *sv = ANJUTA_SOURCEVIEW (user_data);
+	
+	if (!(event_type == GNOME_VFS_MONITOR_EVENT_CHANGED ||
+		  event_type == GNOME_VFS_MONITOR_EVENT_CREATED))
+		return;
+	
+	if (!anjuta_util_diff (anjuta_document_get_uri(sv->priv->document), sv->priv->last_saved_content))
+	{
+		return;
+	}
+	
+	
+	if (strcmp (monitor_uri, info_uri) != 0)
+		return;
+	
+	on_sourceview_uri_changed_prompt(sv);	
+}
+
+static gboolean 
 sourceview_add_monitor(Sourceview* sv)
 {
 	gboolean monitor_enabled = anjuta_preferences_get_int (sv->priv->prefs, MONITOR_KEY);
 
 	if (monitor_enabled)
 	{
-		g_return_if_fail(sv->priv->monitor == NULL);
-		DEBUG_PRINT ("Monitor added for %s", anjuta_document_get_uri(sv->priv->document)); 
+		g_return_val_if_fail(sv->priv->monitor == NULL, FALSE);
+		DEBUG_PRINT ("Monitor added for %s", anjuta_document_get_uri(sv->priv->document)); 	
 		gnome_vfs_monitor_add(&sv->priv->monitor, anjuta_document_get_uri(sv->priv->document),
 							  GNOME_VFS_MONITOR_FILE,
 						  on_sourceview_uri_changed, sv);
 	}
+	return FALSE; /* for g_idle_add */
 }
 
 /* Called when document is loaded completly */
 static void on_document_loaded(AnjutaDocument* doc, GError* err, Sourceview* sv)
 {
+	const gchar *lang;
 	if (err)
 	{
 		anjuta_util_dialog_error(NULL,
@@ -220,6 +236,9 @@ static void on_document_loaded(AnjutaDocument* doc, GError* err, Sourceview* sv)
 	sv->priv->loading = FALSE;
 	
 	sourceview_add_monitor(sv);
+
+	lang = ianjuta_editor_language_get_language(IANJUTA_EDITOR_LANGUAGE(sv), NULL);
+	g_signal_emit_by_name (sv, "language-changed", lang);
 }
 
 /* Show nice progress bar */
@@ -268,19 +287,58 @@ static void on_document_saving(AnjutaDocument    *document,
 	anjuta_status_progress_tick(status, NULL, _("Saving..."));
 }
 
+static gboolean save_if_modified(AnjutaDocument* doc, GtkWindow* parent)
+{
+	GtkWidget* dialog = gtk_message_dialog_new(parent,
+    	GTK_DIALOG_MODAL, GTK_MESSAGE_QUESTION, GTK_BUTTONS_YES_NO, 
+    	_("The file %s was modified by another application. Save it anyway?"), anjuta_document_get_uri_for_display(doc));
+    int result = gtk_dialog_run(GTK_DIALOG(dialog));
+    gtk_widget_destroy(dialog);
+    switch (result)
+    {
+    	case GTK_RESPONSE_YES:
+    	{
+    		return TRUE;
+   		}
+   		default:
+   			return FALSE;
+   	}
+}
+
 /* Called when document is saved completly */
 static void on_document_saved(AnjutaDocument* doc, GError* err, Sourceview* sv)
 {
 	if (err)
 	{
-		anjuta_util_dialog_error(NULL,
-			 "Could not save file: %s", err->message);
+		switch(err->code)
+		{
+			case ANJUTA_DOCUMENT_ERROR_EXTERNALLY_MODIFIED:
+			{
+    			if (save_if_modified(doc, GTK_WINDOW(sv->priv->plugin->shell)))
+    			{
+    				anjuta_document_save(doc, ANJUTA_DOCUMENT_SAVE_IGNORE_MTIME);
+    			}
+    			break;
+			}
+			default:
+			{
+				anjuta_util_dialog_error(NULL,
+				 		"Could not save file %s: %s",anjuta_document_get_uri_for_display(doc),  err->message);
+				break;
+			}
+		}
+	}
+	else
+	{
+		gtk_text_buffer_set_modified(GTK_TEXT_BUFFER(doc), FALSE);
+		g_signal_emit_by_name(G_OBJECT(sv), "save_point", TRUE);
+		/* Set up 2 sec timer */
+		if (sv->priv->monitor_delay > 0)
+ 			g_source_remove (sv->priv->monitor_delay);
+		sv->priv->monitor_delay = g_timeout_add (2000,
+							(GSourceFunc)sourceview_add_monitor, sv);
+		sv->priv->saving = FALSE;
 	}
-	gtk_text_buffer_set_modified(GTK_TEXT_BUFFER(doc), FALSE);
-	g_signal_emit_by_name(G_OBJECT(sv), "save_point",
-						  TRUE);
-	sourceview_add_monitor(sv);
-	sv->priv->saving = FALSE;
 }
 
 static void 
@@ -308,8 +366,6 @@ sourceview_finalize(GObject *object)
 	Sourceview *cobj;
 	cobj = ANJUTA_SOURCEVIEW(object);
 	
-	/* Free private members, etc. */
-	
 	sourceview_remove_monitor(cobj);
 	
 	gtk_widget_destroy(GTK_WIDGET(cobj->priv->tag_window));
@@ -323,18 +379,18 @@ sourceview_finalize(GObject *object)
 
 /* Sync with IANJUTA_MARKABLE_MARKER  */
 
-#define MARKER_PIXMAP_BASIC "marker-basic.png"
-#define MARKER_PIXMAP_LIGHT "marker-light.png"
-#define MARKER_PIXMAP_ATTENTIVE "marker-attentive.png"
-#define MARKER_PIXMAP_INTENSE "marker-intense.png"
+#define MARKER_PIXMAP_LINEMARKER "linemarker.png"
+#define MARKER_PIXMAP_PROGRAM_COUNTER "program-counter.png"
+#define MARKER_PIXMAP_BREAKPOINT_DISABLED "breakpoint-disabled.png"
+#define MARKER_PIXMAP_BREAKPOINT_ENABLED "breakpoint-enabled.png"
 
 #define MARKER_PIXMAP_BOOKMARK "bookmark.png"
 
-#define MARKER_NONE "sv-mark-none"
-#define MARKER_BASIC "sv-mark-basic"
-#define MARKER_LIGHT "sv-mark-light"
-#define MARKER_ATTENTIVE "sv-mark-attentive"
-#define MARKER_INTENSE "sv-mark-intense"
+#define MARKER_NONE "sv-none"
+#define MARKER_LINEMARKER "sv-linemarker"
+#define MARKER_PROGRAM_COUNTER "sv-program-counter"
+#define MARKER_BREAKPOINT_DISABLED "sv-breakpoint-disabled"
+#define MARKER_BREAKPOINT_ENABLED "sv-breakpoint-enabled"
 
 #define MARKER_BOOKMARK "sv-bookmark"
 
@@ -351,48 +407,53 @@ static void sourceview_create_markers(Sourceview* sv)
 	GtkSourceView* view = 	GTK_SOURCE_VIEW(sv->priv->view);
 
 	
-	if ((pixbuf = gdk_pixbuf_new_from_file (PACKAGE_PIXMAPS_DIR"/"MARKER_PIXMAP_BASIC, NULL)))
-	{
-		gtk_source_view_set_marker_pixbuf (view, 
-			MARKER_BASIC, pixbuf);
-		g_object_unref (pixbuf);
-	}
-	if ((pixbuf = gdk_pixbuf_new_from_file (PACKAGE_PIXMAPS_DIR"/"MARKER_PIXMAP_LIGHT, NULL)))
-	{
-		gtk_source_view_set_marker_pixbuf (view, 
-			MARKER_LIGHT, pixbuf);
-		g_object_unref (pixbuf);
-	}
-	if ((pixbuf = gdk_pixbuf_new_from_file (PACKAGE_PIXMAPS_DIR"/"MARKER_PIXMAP_ATTENTIVE, NULL)))
-	{
-		gtk_source_view_set_marker_pixbuf (view, 
-			MARKER_ATTENTIVE, pixbuf);
-		g_object_unref (pixbuf);
-	}
-	if ((pixbuf = gdk_pixbuf_new_from_file (PACKAGE_PIXMAPS_DIR"/"MARKER_PIXMAP_INTENSE, NULL)))
-	{
-		gtk_source_view_set_marker_pixbuf (view, 
-			MARKER_INTENSE, pixbuf);
-		g_object_unref (pixbuf);
-	}
 	if ((pixbuf = gdk_pixbuf_new_from_file (PACKAGE_PIXMAPS_DIR"/"MARKER_PIXMAP_BOOKMARK, NULL)))
 	{
 		gtk_source_view_set_marker_pixbuf (view, 
 			MARKER_BOOKMARK, pixbuf);
 		g_object_unref (pixbuf);
 	}
+	if ((pixbuf = gdk_pixbuf_new_from_file (PACKAGE_PIXMAPS_DIR"/"MARKER_PIXMAP_BREAKPOINT_DISABLED, NULL)))
+	{
+		gtk_source_view_set_marker_pixbuf (view, 
+			MARKER_BREAKPOINT_DISABLED, pixbuf);
+		g_object_unref (pixbuf);
+	}
+	if ((pixbuf = gdk_pixbuf_new_from_file (PACKAGE_PIXMAPS_DIR"/"MARKER_PIXMAP_BREAKPOINT_ENABLED, NULL)))
+	{
+		gtk_source_view_set_marker_pixbuf (view, 
+			MARKER_BREAKPOINT_ENABLED, pixbuf);
+		g_object_unref (pixbuf);
+	}
+	if ((pixbuf = gdk_pixbuf_new_from_file (PACKAGE_PIXMAPS_DIR"/"MARKER_PIXMAP_PROGRAM_COUNTER, NULL)))
+	{
+		gtk_source_view_set_marker_pixbuf (view, 
+			MARKER_PROGRAM_COUNTER, pixbuf);
+		g_object_unref (pixbuf);
+	}
+	if ((pixbuf = gdk_pixbuf_new_from_file (PACKAGE_PIXMAPS_DIR"/"MARKER_PIXMAP_LINEMARKER, NULL)))
+	{
+		gtk_source_view_set_marker_pixbuf (view, 
+			MARKER_LINEMARKER, pixbuf);
+		g_object_unref (pixbuf);
+	}
 }
 
 /* Create tags for highlighting */
 static void sourceview_create_highligth_indic(Sourceview* sv)
 {	
 	sv->priv->important_indic = 
-	          gtk_text_buffer_create_tag (GTK_TEXT_BUFFER(sv->priv->document),
-	          IMPORTANT_INDIC,
-	   		  "foreground", "red", NULL);  
-	//  TO BE DEFINED
-	sv->priv->warning_indic = sv->priv->important_indic; 
-	sv->priv->critical_indic = sv->priv->important_indic;
+		gtk_text_buffer_create_tag (GTK_TEXT_BUFFER(sv->priv->document),
+		IMPORTANT_INDIC,
+		"foreground", "#0000FF", NULL);  
+	sv->priv->warning_indic = 
+		gtk_text_buffer_create_tag (GTK_TEXT_BUFFER(sv->priv->document),
+		WARNING_INDIC,
+		"foreground", "#00FF00", NULL); 
+	sv->priv->critical_indic = 
+		gtk_text_buffer_create_tag (GTK_TEXT_BUFFER(sv->priv->document),
+		CRITICAL_INDIC,
+		"foreground", "#FF0000", "underline", PANGO_UNDERLINE_ERROR, NULL);
 }
 
 
@@ -423,10 +484,9 @@ sourceview_new(const gchar* uri, const gchar* filename, AnjutaPlugin* plugin)
 					 
 	/* Create View instance */
 	sv->priv->view = ANJUTA_VIEW(anjuta_view_new(sv->priv->document));
-	g_signal_connect(G_OBJECT(sv->priv->view), "char_added",
+	g_signal_connect_after(G_OBJECT(sv->priv->view), "char_added",
 					G_CALLBACK(on_document_char_added), sv);
 	gtk_source_view_set_smart_home_end(GTK_SOURCE_VIEW(sv->priv->view), FALSE);
-	
 	sv->priv->tag_window = sourceview_tags_new(plugin);
 	sv->priv->autocomplete = sourceview_autocomplete_new();
 	sv->priv->args = sourceview_args_new(plugin, sv->priv->view);
@@ -436,6 +496,9 @@ sourceview_new(const gchar* uri, const gchar* filename, AnjutaPlugin* plugin)
 	anjuta_view_register_completion(sv->priv->view, TAG_WINDOW(sv->priv->scope));
 	anjuta_view_register_completion(sv->priv->view, TAG_WINDOW(sv->priv->autocomplete));
 	
+	/* VFS monitor */
+	sv->priv->last_saved_content = NULL;
+	
 	/* Apply Preferences */
 	g_object_get(G_OBJECT(plugin), "shell", &shell, NULL);
 	sv->priv->prefs = anjuta_shell_get_preferences(shell, NULL);
@@ -468,10 +531,12 @@ sourceview_new(const gchar* uri, const gchar* filename, AnjutaPlugin* plugin)
 /* IAnjutaFile interface */
 
 /* Open uri in Editor */
-static void 
+static void
 ifile_open (IAnjutaFile* file, const gchar *uri, GError** e)
 {
 	Sourceview* sv = ANJUTA_SOURCEVIEW(file);
+	// const gchar* lang;
+	// IAnjutaLanguageSupport* lang_support;
 	sourceview_remove_monitor(sv);
 	anjuta_document_load(sv->priv->document, uri, NULL,
 						 -1, FALSE);
@@ -495,19 +560,26 @@ ifile_savable_save (IAnjutaFileSavable* file, GError** e)
 	Sourceview* sv = ANJUTA_SOURCEVIEW(file);
 	sourceview_remove_monitor(sv);
 					 
-	anjuta_document_save(sv->priv->document, ANJUTA_DOCUMENT_SAVE_IGNORE_BACKUP);
+	anjuta_document_save(sv->priv->document, 0);
 }
 
 /* Save file as */
 static void 
 ifile_savable_save_as (IAnjutaFileSavable* file, const gchar *uri, GError** e)
 {
+	GtkTextIter start_iter;
+	GtkTextIter end_iter;
 	Sourceview* sv = ANJUTA_SOURCEVIEW(file);
 	sourceview_remove_monitor(sv);
 	/* TODO: Set correct encoding */
+	gtk_text_buffer_get_bounds (GTK_TEXT_BUFFER(sv->priv->document),
+								&start_iter, &end_iter);
+	g_free(sv->priv->last_saved_content);
+	sv->priv->last_saved_content = gtk_text_buffer_get_slice (
+															  GTK_TEXT_BUFFER(sv->priv->document),
+															  &start_iter, &end_iter, TRUE);
 	anjuta_document_save_as(sv->priv->document, 
-							uri, anjuta_encoding_get_current(),
-							ANJUTA_DOCUMENT_SAVE_IGNORE_BACKUP);
+							uri, anjuta_encoding_get_current(), 0);
 	if (sv->priv->filename)
 	{
 		g_free(sv->priv->filename);
@@ -548,6 +620,49 @@ ifile_iface_init (IAnjutaFileIface *iface)
 
 /* IAnjutaEditor interface */
 
+/* Grab focus */
+static void ieditor_grab_focus (IAnjutaEditor *editor, GError **e)
+{
+	gtk_widget_grab_focus (GTK_WIDGET (ANJUTA_SOURCEVIEW (editor)->priv->view));
+}
+
+static gint
+ieditor_get_tab_size (IAnjutaEditor *editor, GError **e)
+{
+	Sourceview* sv = ANJUTA_SOURCEVIEW(editor);
+	return gtk_source_view_get_tabs_width (GTK_SOURCE_VIEW (sv->priv->view));
+}
+
+static void
+ieditor_set_tab_size (IAnjutaEditor *editor, gint tabsize, GError **e)
+{
+	Sourceview* sv = ANJUTA_SOURCEVIEW(editor);
+	gtk_source_view_set_tabs_width(GTK_SOURCE_VIEW(sv->priv->view), tabsize);
+}
+
+static gboolean
+ieditor_get_use_spaces (IAnjutaEditor *editor, GError **e)
+{
+	Sourceview* sv = ANJUTA_SOURCEVIEW(editor);
+	return gtk_source_view_get_insert_spaces_instead_of_tabs (GTK_SOURCE_VIEW(sv->priv->view));
+}
+
+static void
+ieditor_set_use_spaces (IAnjutaEditor *editor, gboolean use_spaces, GError **e)
+{
+	Sourceview* sv = ANJUTA_SOURCEVIEW(editor);
+	gtk_source_view_set_insert_spaces_instead_of_tabs(GTK_SOURCE_VIEW(sv->priv->view), use_spaces);
+}
+
+static void
+ieditor_set_auto_indent (IAnjutaEditor *editor, gboolean auto_indent, GError **e)
+{
+	Sourceview* sv = ANJUTA_SOURCEVIEW(editor);
+	DEBUG_PRINT("Setting auto-indent to %d", auto_indent);
+	gtk_source_view_set_auto_indent(GTK_SOURCE_VIEW(sv->priv->view), auto_indent);
+}
+
+
 /* Scroll to line */
 static void ieditor_goto_line(IAnjutaEditor *editor, gint line, GError **e)
 {
@@ -565,32 +680,38 @@ static void ieditor_goto_line(IAnjutaEditor *editor, gint line, GError **e)
 /* Scroll to position */
 static void ieditor_goto_position(IAnjutaEditor *editor, gint position, GError **e)
 {
+	GtkTextIter iter;
+	
 	Sourceview* sv = ANJUTA_SOURCEVIEW(editor);
-	anjuta_document_goto_line(sv->priv->document, 
-							 ianjuta_editor_get_line_from_position(editor, position, NULL));
+	gtk_text_buffer_get_iter_at_offset(GTK_TEXT_BUFFER(sv->priv->document),
+									   &iter, position);
+	gtk_text_buffer_place_cursor (GTK_TEXT_BUFFER (sv->priv->document), &iter);
+	gtk_text_view_scroll_to_iter (GTK_TEXT_VIEW (sv->priv->view),
+								  &iter, 0, FALSE, 0, 0);
 }
 
 /* Return a newly allocated pointer containing the whole text */
-static gchar* ieditor_get_text(IAnjutaEditor* editor, gint start,
-							   gint end, GError **e)
+static gchar* ieditor_get_text(IAnjutaEditor* editor, gint position,
+							   gint length, GError **e)
 {
 	GtkTextIter start_iter;
 	GtkTextIter end_iter;
 	Sourceview* sv = ANJUTA_SOURCEVIEW(editor);
 	
-	gtk_text_buffer_get_iter_at_offset(GTK_TEXT_BUFFER(sv->priv->document),
-								   &start_iter, start);
-	gtk_text_buffer_get_iter_at_offset(GTK_TEXT_BUFFER(sv->priv->document),
-								   &end_iter, end);
-	return gtk_text_buffer_get_text(GTK_TEXT_BUFFER(sv->priv->document),
-									&start_iter, &end_iter, FALSE);
-}
+	g_return_val_if_fail (position >= 0, NULL);
+	if (length == 0)
+		return NULL;
 
-/* TODO! No documentation, so we do nothing */
-static gchar* ieditor_get_attributes(IAnjutaEditor* editor, 
-									 gint start, gint end, GError **e)
-{
-	return NULL;
+	gtk_text_buffer_get_iter_at_offset(GTK_TEXT_BUFFER(sv->priv->document),
+								   &start_iter, position);
+	if (length > 0)
+		gtk_text_buffer_get_iter_at_offset(GTK_TEXT_BUFFER(sv->priv->document),
+										   &end_iter, position + length);
+	else
+		gtk_text_buffer_get_iter_at_offset(GTK_TEXT_BUFFER(sv->priv->document),
+										   &end_iter, -1);
+	return gtk_text_buffer_get_slice(GTK_TEXT_BUFFER(sv->priv->document),
+									&start_iter, &end_iter, TRUE);
 }
 
 /* Get cursor position */
@@ -635,8 +756,8 @@ static gint ieditor_get_length(IAnjutaEditor *editor, GError **e)
 								   &start_iter);
 	gtk_text_buffer_get_end_iter(GTK_TEXT_BUFFER(sv->priv->document),
 								   &end_iter);
-	text = gtk_text_buffer_get_text(GTK_TEXT_BUFFER(sv->priv->document),
-									&start_iter, &end_iter, FALSE);
+	text = gtk_text_buffer_get_slice(GTK_TEXT_BUFFER(sv->priv->document),
+									&start_iter, &end_iter, TRUE);
 	length = g_utf8_strlen(text,  -1);
 	g_free(text);
 
@@ -676,6 +797,25 @@ static void ieditor_append(IAnjutaEditor *editor, const gchar* text,
 						   &iter, text, length);
 }
 
+static void ieditor_erase(IAnjutaEditor* editor, gint position, gint length, GError **e)
+{
+	GtkTextIter start, end;
+	Sourceview* sv = ANJUTA_SOURCEVIEW(editor);
+	GtkTextBuffer* buffer = GTK_TEXT_BUFFER(sv->priv->document);
+
+	g_return_if_fail (position >= 0);
+	if (length == 0)
+		return;
+
+	gtk_text_buffer_get_iter_at_offset(buffer, &start, position);
+
+	if (length > 0)
+		gtk_text_buffer_get_iter_at_offset(buffer, &end, position + length);
+	else
+		gtk_text_buffer_get_iter_at_offset(buffer, &end, -1);
+	gtk_text_buffer_delete (buffer, &start, &end);
+}
+
 static void ieditor_erase_all(IAnjutaEditor *editor, GError **e)
 {
 	Sourceview* sv = ANJUTA_SOURCEVIEW(editor);
@@ -692,10 +832,24 @@ static gboolean ieditor_can_redo(IAnjutaEditor *editor, GError **e)
 /* Return true if editor can undo */
 static gboolean ieditor_can_undo(IAnjutaEditor *editor, GError **e)
 {
-	Sourceview* sv = ANJUTA_SOURCEVIEW(editor);
+	Sourceview* sv = ANJUTA_SOURCEVIEW(editor);	
 	return gtk_source_buffer_can_undo(GTK_SOURCE_BUFFER(sv->priv->document));
 }
 
+/* Return true if editor can undo */
+static void ieditor_begin_undo_action (IAnjutaEditor *editor, GError **e)
+{
+	Sourceview* sv = ANJUTA_SOURCEVIEW(editor);	
+	gtk_text_buffer_begin_user_action (GTK_TEXT_BUFFER(sv->priv->document));
+}
+
+/* Return true if editor can undo */
+static void ieditor_end_undo_action (IAnjutaEditor *editor, GError **e)
+{
+	Sourceview* sv = ANJUTA_SOURCEVIEW(editor);	
+	gtk_text_buffer_end_user_action (GTK_TEXT_BUFFER(sv->priv->document));
+}
+
 /* Return column of cursor */
 static gint ieditor_get_column(IAnjutaEditor *editor, GError **e)
 {
@@ -745,16 +899,42 @@ static gint ieditor_get_line_from_position(IAnjutaEditor *editor,
 									   &iter, position);
 
 	line = gtk_text_iter_get_line(&iter) + 1;
-	line = line ? line - 1 : 0;
+	/* line = line ? line - 1 : 0; */
 	
 	return line;
 }
 
+static gint ieditor_get_line_begin_position(IAnjutaEditor *editor,
+											gint line, GError **e)
+{
+	GtkTextIter iter;
+	Sourceview* sv = ANJUTA_SOURCEVIEW(editor);
+	
+	gtk_text_buffer_get_iter_at_line_offset (GTK_TEXT_BUFFER(sv->priv->document),
+											 &iter, line - 1, 0);
+	return gtk_text_iter_get_offset (&iter);
+}
+
+static gint ieditor_get_line_end_position(IAnjutaEditor *editor,
+											gint line, GError **e)
+{
+	GtkTextIter iter;
+	Sourceview* sv = ANJUTA_SOURCEVIEW(editor);
+	
+	gtk_text_buffer_get_iter_at_line(GTK_TEXT_BUFFER(sv->priv->document),
+									 &iter, line - 1);
+	/* If iter is not at line end, move it */
+	if (!gtk_text_iter_ends_line(&iter))
+		gtk_text_iter_forward_to_line_end (&iter);
+	return gtk_text_iter_get_offset(&iter);
+}
+
 static void 
 ieditor_undo(IAnjutaEditor* edit, GError** ee)
 {
 	Sourceview* sv = ANJUTA_SOURCEVIEW(edit);
-	gtk_source_buffer_undo(GTK_SOURCE_BUFFER(sv->priv->document));
+	if (ieditor_can_undo(edit, NULL))	
+		gtk_source_buffer_undo(GTK_SOURCE_BUFFER(sv->priv->document));
 	anjuta_view_scroll_to_cursor(sv->priv->view);
 	g_signal_emit_by_name(G_OBJECT(sv), "update_ui", sv); 
 }
@@ -763,33 +943,59 @@ static void
 ieditor_redo(IAnjutaEditor* edit, GError** ee)
 {
 	Sourceview* sv = ANJUTA_SOURCEVIEW(edit);
-	gtk_source_buffer_redo(GTK_SOURCE_BUFFER(sv->priv->document));
+	if (ieditor_can_redo(edit, NULL))	
+		gtk_source_buffer_redo(GTK_SOURCE_BUFFER(sv->priv->document));
 	anjuta_view_scroll_to_cursor(sv->priv->view);
 }
 
+static IAnjutaIterable*
+ieditor_get_cell_iter(IAnjutaEditor* edit, gint position, GError** e)
+{
+	Sourceview* sv = ANJUTA_SOURCEVIEW(edit);
+	GtkTextBuffer* buffer = GTK_TEXT_BUFFER(sv->priv->document);
+	GtkTextIter iter;
+	SourceviewCell* cell;
+	
+	gtk_text_buffer_get_iter_at_offset(buffer, &iter, position);
+	cell = sourceview_cell_new(&iter, GTK_TEXT_VIEW(sv->priv->view));
+	
+	return IANJUTA_ITERABLE(cell);
+}
+
 static void
 ieditor_iface_init (IAnjutaEditorIface *iface)
 {
+	iface->grab_focus = ieditor_grab_focus;
+	iface->get_tabsize = ieditor_get_tab_size;
+	iface->set_tabsize = ieditor_set_tab_size;
+	iface->get_use_spaces = ieditor_get_use_spaces;
+	iface->set_use_spaces = ieditor_set_use_spaces;
+    iface->set_auto_indent = ieditor_set_auto_indent;
 	iface->goto_line = ieditor_goto_line;
 	iface->goto_position = ieditor_goto_position;
 	iface->get_text = ieditor_get_text;
-	iface->get_attributes = ieditor_get_attributes;
 	iface->get_position = ieditor_get_position;
 	iface->get_lineno = ieditor_get_lineno;
 	iface->get_length = ieditor_get_length;
 	iface->get_current_word = ieditor_get_current_word;
 	iface->insert = ieditor_insert;
 	iface->append = ieditor_append;
+	iface->erase = ieditor_erase;
 	iface->erase_all = ieditor_erase_all;
 	iface->get_filename = ieditor_get_filename;
 	iface->can_undo = ieditor_can_undo;
 	iface->can_redo = ieditor_can_redo;
+	iface->begin_undo_action = ieditor_begin_undo_action;
+	iface->end_undo_action = ieditor_end_undo_action;
 	iface->get_column = ieditor_get_column;
 	iface->get_overwrite = ieditor_get_overwrite;
 	iface->set_popup_menu = ieditor_set_popup_menu;
 	iface->get_line_from_position = ieditor_get_line_from_position;
 	iface->undo = ieditor_undo;
 	iface->redo = ieditor_redo;
+	iface->get_cell_iter = ieditor_get_cell_iter;
+	iface->get_line_begin_position = ieditor_get_line_begin_position;
+	iface->get_line_end_position = ieditor_get_line_end_position;	
 }
 
 static void
@@ -799,8 +1005,9 @@ set_select(Sourceview* sv, GtkTextIter* start_iter, GtkTextIter* end_iter)
 									  "insert", start_iter);
 	gtk_text_buffer_move_mark_by_name(GTK_TEXT_BUFFER(sv->priv->document),
 									  "selection_bound", end_iter);
-	gtk_text_view_scroll_mark_onscreen(GTK_TEXT_VIEW(sv->priv->view),
-		 gtk_text_buffer_get_insert(GTK_TEXT_BUFFER(sv->priv->document)));			
+	gtk_text_view_scroll_to_mark(GTK_TEXT_VIEW(sv->priv->view),
+								 gtk_text_buffer_get_insert(GTK_TEXT_BUFFER(sv->priv->document)),
+								 0,  FALSE, 0, 0);			
 }
 
 /* IAnjutaEditorSelection */
@@ -837,8 +1044,9 @@ iselect_block(IAnjutaEditorSelection* edit, GError** e)
 								   &start_iter);
 	gtk_text_buffer_get_end_iter(GTK_TEXT_BUFFER(sv->priv->document),
 								   &end_iter);
-	text = gtk_text_buffer_get_text(GTK_TEXT_BUFFER(sv->priv->document),
-										&start_iter, &end_iter, FALSE);
+	text = gtk_text_buffer_get_slice
+	(GTK_TEXT_BUFFER(sv->priv->document),
+										&start_iter, &end_iter, TRUE);
 	if (text)
 	{
 		gboolean found = FALSE;
@@ -915,8 +1123,8 @@ static gchar* iselect_get(IAnjutaEditorSelection* editor, GError **e)
 	if (gtk_text_buffer_get_selection_bounds(GTK_TEXT_BUFFER(sv->priv->document),
 										 &start_iter, &end_iter))
 	{
-		return gtk_text_buffer_get_text(GTK_TEXT_BUFFER(sv->priv->document),
-										&start_iter, &end_iter, FALSE);
+		return gtk_text_buffer_get_slice(GTK_TEXT_BUFFER(sv->priv->document),
+										&start_iter, &end_iter, TRUE);
 	}
 	else
 		return NULL;
@@ -995,17 +1203,17 @@ static void
 iselect_clear(IAnjutaEditorSelection* edit, GError** ee)
 {
 	Sourceview* sv = ANJUTA_SOURCEVIEW(edit);
-	if (_anjuta_document_get_has_selection(sv->priv->document))
+	if (gtk_text_buffer_get_has_selection(GTK_TEXT_BUFFER(sv->priv->document)))
 		anjuta_view_delete_selection(sv->priv->view);
 	else
 	{
 		GtkTextBuffer* buffer = GTK_TEXT_BUFFER(sv->priv->document);
-		GtkTextIter start, end;
-		gtk_text_buffer_get_iter_at_mark(buffer, &start, gtk_text_buffer_get_insert(buffer));
-		gtk_text_buffer_get_iter_at_mark(buffer, &end, gtk_text_buffer_get_insert(buffer));
-		gtk_text_iter_forward_char(&end);
-		 
-		gtk_text_buffer_delete(buffer, &start, &end);
+		GtkTextIter cursor;
+		gtk_text_buffer_get_iter_at_mark(buffer, &cursor, gtk_text_buffer_get_insert(buffer));
+		
+		/* Fix #388731 */
+		gtk_text_iter_forward_char(&cursor);
+		gtk_text_buffer_backspace(buffer, &cursor, TRUE, TRUE);
 	}
 }
 
@@ -1082,15 +1290,6 @@ static void iassist_iface_init(IAnjutaEditorAssistIface* iface)
 	iface->autocomplete = iassist_autocomplete;
 }
 
-/* Marker Struture to connect handles and markers */
-typedef struct
-{
-	gint handle;
-	GtkSourceMarker* marker;
-	gint location;
-	IAnjutaMarkableMarker type;
-} SVMarker;
-
 static gint
 imark_mark(IAnjutaMarkable* mark, gint location, IAnjutaMarkableMarker marker,
 		   GError **e)
@@ -1099,42 +1298,40 @@ imark_mark(IAnjutaMarkable* mark, gint location, IAnjutaMarkableMarker marker,
 	GtkTextIter iter;
 	GtkSourceMarker* source_marker;
 	gchar* name;
+	static gint marker_count = 0;
 	
 	gtk_text_buffer_get_iter_at_line(GTK_TEXT_BUFFER(sv->priv->document),
 									 &iter, location  - 1);
 	switch (marker)
 	{
-		case IANJUTA_MARKABLE_NONE:
-			name = MARKER_NONE;
+		case IANJUTA_MARKABLE_LINEMARKER:
+			name = MARKER_LINEMARKER;
 			break;
-		case IANJUTA_MARKABLE_BASIC:
-			name = MARKER_BASIC;
+		case IANJUTA_MARKABLE_PROGRAM_COUNTER:
+			name = MARKER_PROGRAM_COUNTER;
 			break;
-		case IANJUTA_MARKABLE_LIGHT:
-			name = MARKER_LIGHT;
+		case IANJUTA_MARKABLE_BOOKMARK:
+			name = MARKER_BOOKMARK;
 			break;
-		case IANJUTA_MARKABLE_INTENSE:
-			name = MARKER_INTENSE;
+		case IANJUTA_MARKABLE_BREAKPOINT_DISABLED:
+			name = MARKER_BREAKPOINT_DISABLED;
 			break;
-		case IANJUTA_MARKABLE_ATTENTIVE:
-			name = MARKER_ATTENTIVE;
+		case IANJUTA_MARKABLE_BREAKPOINT_ENABLED:
+			name = MARKER_BREAKPOINT_ENABLED;
 			break;
 		default:
-			DEBUG_PRINT("Unkonown marker type: %d!", marker);
+			DEBUG_PRINT("Unknown marker type: %d!", marker);
 			name = MARKER_NONE;
 	}
 		
 	source_marker = gtk_source_buffer_create_marker(GTK_SOURCE_BUFFER(sv->priv->document), 
 													NULL, name, &iter);
-	SVMarker* sv_marker = g_new0(SVMarker, 1);
-	sv_marker->handle = sv->priv->marker_count++;
-	sv_marker->marker = source_marker;
-	sv_marker->location = location;
-	sv_marker->type = marker;
+
+	g_object_set_data(G_OBJECT(source_marker), "handle", GINT_TO_POINTER(marker_count++));
+	g_object_set_data(G_OBJECT(source_marker), "type", GINT_TO_POINTER(marker));
+	g_object_set_data(G_OBJECT(source_marker), "location", GINT_TO_POINTER(location));
 	
-	sv->priv->markers = g_list_append(sv->priv->markers, sv_marker);
-	
-	return sv_marker->handle;
+	return marker_count;
 }
 
 static void
@@ -1142,19 +1339,16 @@ imark_unmark(IAnjutaMarkable* mark, gint location, IAnjutaMarkableMarker marker,
 			 GError **e)
 {
 	Sourceview* sv = ANJUTA_SOURCEVIEW(mark);
-	GList* node = sv->priv->markers;
-	while (node)
+	GtkSourceBuffer* buffer = GTK_SOURCE_BUFFER(sv->priv->document);
+	GtkSourceMarker* smarker;
+	for (smarker = gtk_source_buffer_get_first_marker(buffer);
+		 smarker != NULL; smarker = gtk_source_marker_next(smarker))
 	{
-		SVMarker* sv_marker = node->data;
-		if (sv_marker->location == location)
-		{
-			gtk_source_buffer_delete_marker(GTK_SOURCE_BUFFER(sv->priv->document),
-											sv_marker->marker);
-			sv->priv->markers = g_list_remove(sv->priv->markers, node);
-			return;
-		}
-		node = g_list_next(node);
-	}	
+		int marker_location = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(smarker), "location"));
+		if (marker_location == location)
+			gtk_source_buffer_delete_marker(buffer, smarker);
+		/* We do not break here because there might be multiple markers at this location */
+	}
 }
 
 static gboolean
@@ -1162,15 +1356,14 @@ imark_is_marker_set(IAnjutaMarkable* mark, gint location,
 					IAnjutaMarkableMarker marker, GError **e)
 {
 	Sourceview* sv = ANJUTA_SOURCEVIEW(mark);
-	GList* node = sv->priv->markers;
-	while (node)
+	GtkSourceBuffer* buffer = GTK_SOURCE_BUFFER(sv->priv->document);
+	GtkSourceMarker* smarker;
+	for (smarker = gtk_source_buffer_get_first_marker(buffer);
+		 smarker != NULL; smarker = gtk_source_marker_next(smarker))
 	{
-		SVMarker* sv_marker = node->data;
-		if (sv_marker->location == location)
-		{
+		int marker_location = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(smarker), "location"));
+		if (marker_location == location)
 			return TRUE;
-		}
-		node = g_list_next(node);
 	}
 	return FALSE;
 }
@@ -1179,17 +1372,16 @@ static gint
 imark_location_from_handle(IAnjutaMarkable* mark, gint handle, GError **e)
 {
 	Sourceview* sv = ANJUTA_SOURCEVIEW(mark);
-	GList* node = sv->priv->markers;
-	while (node)
+	GtkSourceBuffer* buffer = GTK_SOURCE_BUFFER(sv->priv->document);
+	GtkSourceMarker* marker;
+	for (marker = gtk_source_buffer_get_first_marker(buffer);
+		 marker != NULL; marker = gtk_source_marker_next(marker))
 	{
-		SVMarker* sv_marker = node->data;
-		if (sv_marker->handle == handle)
-		{
-			return sv_marker->location;
-		}
-		node = g_list_next(node);
+		int marker_handle = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(marker), "handle"));
+		if (marker_handle == handle)
+			return GPOINTER_TO_INT(g_object_get_data(G_OBJECT(marker), "location"));
 	}
-	return FALSE;
+	return 0;
 }
 
 static void
@@ -1197,18 +1389,15 @@ imark_delete_all_markers(IAnjutaMarkable* mark, IAnjutaMarkableMarker marker,
 						 GError **e)
 {
 	Sourceview* sv = ANJUTA_SOURCEVIEW(mark);
-	GList* node = sv->priv->markers;
-	while (node)
+	GtkSourceBuffer* buffer = GTK_SOURCE_BUFFER(sv->priv->document);
+	GtkSourceMarker* smarker;
+	for (smarker = gtk_source_buffer_get_first_marker(buffer);
+		 smarker != NULL; smarker = gtk_source_marker_next(smarker))
 	{
-		SVMarker* sv_marker = node->data;
-		if (sv_marker->type == marker)
-		{
-			gtk_source_buffer_delete_marker(GTK_SOURCE_BUFFER(sv->priv->document),
-											sv_marker->marker);
-			sv->priv->markers = g_list_delete_link(sv->priv->markers, node);
-			node = sv->priv->markers;
-		}
-		node = g_list_next(node);
+		IAnjutaMarkableMarker type = 
+			GPOINTER_TO_INT(g_object_get_data(G_OBJECT(smarker), "type"));
+		if (type == marker)
+			gtk_source_buffer_delete_marker(buffer, smarker);
 	}
 }
 
@@ -1238,7 +1427,12 @@ iindic_clear (IAnjutaIndicable *indic, GError **e)
 	gtk_text_buffer_remove_tag_by_name (GTK_TEXT_BUFFER(sv->priv->document),
 	                                     IMPORTANT_INDIC,
 	                                     &start, &end);
-	// Remove warning_indic, critical_indic
+	gtk_text_buffer_remove_tag_by_name (GTK_TEXT_BUFFER(sv->priv->document),
+	                                     WARNING_INDIC,
+	                                     &start, &end);
+	gtk_text_buffer_remove_tag_by_name (GTK_TEXT_BUFFER(sv->priv->document),
+	                                     CRITICAL_INDIC,
+	                                     &start, &end);
 }
 
 static void
@@ -1473,6 +1667,85 @@ iprint_iface_init(IAnjutaPrintIface* iface)
 	iface->print_preview = iprint_print_preview;
 }
 
+static const GList*
+ilanguage_get_supported_languages (IAnjutaEditorLanguage *ilanguage,
+								   GError **err)
+{
+	GtkSourceLanguagesManager* manager = gtk_source_languages_manager_new();
+	const GSList* manager_languages = gtk_source_languages_manager_get_available_languages(manager);
+	GList* languages = NULL;
+	
+	while (manager_languages)
+	{
+		languages = g_list_append(languages, gtk_source_language_get_name(
+			GTK_SOURCE_LANGUAGE(manager_languages->data)));
+		manager_languages = g_slist_next(manager_languages);
+	}
+	return languages;
+}
+
+static const gchar*
+ilanguage_get_language_name (IAnjutaEditorLanguage *ilanguage,
+							 const gchar *language, GError **err)
+{	
+	return language;
+}
+
+static void
+ilanguage_set_language (IAnjutaEditorLanguage *ilanguage,
+						const gchar *language, GError **err)
+{
+	Sourceview* sv = ANJUTA_SOURCEVIEW(ilanguage);
+	GtkSourceBuffer* buffer = GTK_SOURCE_BUFFER(sv->priv->document);
+	GtkSourceLanguagesManager* manager = gtk_source_languages_manager_new();
+	const GSList* langs = gtk_source_languages_manager_get_available_languages(manager);
+	
+	while (langs)
+	{
+		gchar* name = gtk_source_language_get_name(GTK_SOURCE_LANGUAGE(langs->data));
+		if (g_str_equal(name, language))
+		{
+			gtk_source_buffer_set_language(buffer, GTK_SOURCE_LANGUAGE(langs->data));
+			g_signal_emit_by_name (ilanguage, "language-changed", language);
+			return;
+		}
+		langs = g_slist_next(langs);
+	}
+	/* Language not found -> use autodetection */
+	{
+		gchar* mime_type = anjuta_document_get_mime_type(ANJUTA_DOCUMENT(buffer));
+		GtkSourceLanguage* lang = gtk_source_languages_manager_get_language_from_mime_type(
+			manager, mime_type);
+		if (lang != NULL)
+		{
+			gtk_source_buffer_set_language(buffer, lang);
+			g_signal_emit_by_name (ilanguage, "language-changed", 
+				gtk_source_language_get_name(lang));
+		}
+	}	
+}
+
+static const gchar*
+ilanguage_get_language (IAnjutaEditorLanguage *ilanguage, GError **err)
+{
+	Sourceview* sv = ANJUTA_SOURCEVIEW(ilanguage);
+	GtkSourceBuffer* buffer = GTK_SOURCE_BUFFER(sv->priv->document);
+	GtkSourceLanguage* lang = gtk_source_buffer_get_language(buffer);
+	if (lang)
+		return gtk_source_language_get_name(lang);
+	else
+		return NULL;
+}
+
+static void
+ilanguage_iface_init (IAnjutaEditorLanguageIface *iface)
+{
+	iface->get_supported_languages = ilanguage_get_supported_languages;
+	iface->get_language_name = ilanguage_get_language_name;
+	iface->get_language = ilanguage_get_language;
+	iface->set_language = ilanguage_set_language;
+}
+
 ANJUTA_TYPE_BEGIN(Sourceview, sourceview, GTK_TYPE_SCROLLED_WINDOW);
 ANJUTA_TYPE_ADD_INTERFACE(ifile, IANJUTA_TYPE_FILE);
 ANJUTA_TYPE_ADD_INTERFACE(isavable, IANJUTA_TYPE_FILE_SAVABLE);
@@ -1484,4 +1757,5 @@ ANJUTA_TYPE_ADD_INTERFACE(iassist, IANJUTA_TYPE_EDITOR_ASSIST);
 ANJUTA_TYPE_ADD_INTERFACE(iconvert, IANJUTA_TYPE_EDITOR_CONVERT);
 ANJUTA_TYPE_ADD_INTERFACE(ibookmark, IANJUTA_TYPE_BOOKMARK);
 ANJUTA_TYPE_ADD_INTERFACE(iprint, IANJUTA_TYPE_PRINT);
+ANJUTA_TYPE_ADD_INTERFACE(ilanguage, IANJUTA_TYPE_EDITOR_LANGUAGE);
 ANJUTA_TYPE_END;
